@@ -1,11 +1,12 @@
 package main
 
 import (
+	"errors"
 	"flag"
 	"fmt"
 	"io"
-	"io/ioutil"
 	"log"
+	"mime"
 	"net/http"
 	"net/url"
 	"os"
@@ -31,6 +32,8 @@ const (
 
 var wrapAndSort string
 
+var errUnsupportedHoster = errors.New("unsupported hoster")
+
 func passthroughEnv() []string {
 	var relevantVariables = []string{
 		"HOME",
@@ -130,33 +133,47 @@ func (u *upstream) get(gopath, repo, rev string) error {
 	u.rr = rr
 	dir := filepath.Join(gopath, "src", rr.Root)
 	if rev != "" {
+		// Run "git clone {repo} {dir}" and "git checkout {tag}"
 		return rr.VCS.CreateAtRev(dir, rr.Repo, rev)
 	}
+	// Run "git clone {repo} {dir}" (or the equivalent command for hg, svn, bzr)
 	return rr.VCS.Create(dir, rr.Repo)
 }
 
-func (u *upstream) tarballFromHoster() error {
-	var tarURL string
+func (u *upstream) tarballUrl() (string, error) {
 	repo := strings.TrimSuffix(u.rr.Repo, ".git")
 	repoU, err := url.Parse(repo)
 	if err != nil {
-		return fmt.Errorf("parse URL: %w", err)
+		return "", fmt.Errorf("parse URL: %w", err)
 	}
 
 	switch repoU.Host {
 	case "github.com":
-		tarURL = fmt.Sprintf("%s/archive/%s.tar.%s",
-			repo, u.tag, u.compression)
-	case "gitlab.com":
+		return fmt.Sprintf("%s/archive/%s.tar.%s",
+			repo, u.tag, u.compression), nil
+	case "gitlab.com", "salsa.debian.org":
 		parts := strings.Split(repoU.Path, "/")
 		if len(parts) < 3 {
-			return fmt.Errorf("incomplete repo URL: %s", u.rr.Repo)
+			return "", fmt.Errorf("incomplete repo URL: %s", u.rr.Repo)
 		}
 		project := parts[2]
-		tarURL = fmt.Sprintf("%s/-/archive/%s/%s-%s.tar.%s",
-			repo, u.tag, project, u.tag, u.compression)
+		return fmt.Sprintf("%s/-/archive/%s/%s-%s.tar.%s",
+			repo, u.tag, project, u.tag, u.compression), nil
+	case "git.sr.ht":
+		return fmt.Sprintf("%s/archive/%s.tar.%s",
+			repo, u.tag, u.compression), nil
+	case "codeberg.org":
+		return fmt.Sprintf("%s/archive/%s.tar.%s",
+			repo, u.tag, u.compression), nil
 	default:
-		return fmt.Errorf("unsupported hoster") // Don't change
+		return "", errUnsupportedHoster
+	}
+}
+
+func (u *upstream) tarballFromHoster() error {
+	tarURL, err := u.tarballUrl()
+	if err != nil {
+		return err
 	}
 
 	done := make(chan struct{})
@@ -171,7 +188,7 @@ func (u *upstream) tarballFromHoster() error {
 }
 
 func (u *upstream) tar(gopath, repo string) error {
-	f, err := ioutil.TempFile("", "dh-make-golang")
+	f, err := os.CreateTemp("", "dh-make-golang")
 	if err != nil {
 		return fmt.Errorf("create temp file: %w", err)
 	}
@@ -179,18 +196,18 @@ func (u *upstream) tar(gopath, repo string) error {
 	f.Close()
 
 	if u.isRelease {
-		if !u.hasGodeps { // No need to repack; fetch pristine tarball
+		if u.hasGodeps {
+			log.Printf("Godeps/_workspace exists, not downloading tarball from hoster.")
+		} else {
 			u.compression = "gz"
-			if err := u.tarballFromHoster(); err != nil {
-				if err.Error() == "unsupported hoster" {
-					log.Printf("INFO: Hoster does not provide release tarball\n")
-				} else {
-					return fmt.Errorf("tarball from hoster: %w", err)
-				}
+			if err := u.tarballFromHoster(); err == nil {
+				return nil
+			} else if err == errUnsupportedHoster {
+				log.Printf("INFO: Hoster does not provide release tarball\n")
+			} else {
+				return fmt.Errorf("tarball from hoster: %w", err)
 			}
-			return nil
 		}
-		log.Printf("Godeps/_workspace exists, not downloading tarball from hoster.")
 	}
 
 	u.compression = "xz"
@@ -213,26 +230,24 @@ func (u *upstream) tar(gopath, repo string) error {
 // findMains finds main packages within the repo (useful to auto-detect the
 // package type).
 func (u *upstream) findMains(gopath, repo string) error {
-	cmd := exec.Command("go", "list", "-f", "{{.ImportPath}} {{.Name}}", repo+"/...")
+	cmd := exec.Command("go", "list", "-e", "-f", "{{.ImportPath}} {{.Name}}", repo+"/...")
+	cmd.Dir = filepath.Join(gopath, "src", repo)
+	cmd.Env = passthroughEnv()
 	cmd.Stderr = os.Stderr
-	cmd.Env = append([]string{
-		"GO111MODULE=off",
-		"GOPATH=" + gopath,
-	}, passthroughEnv()...)
-
+	log.Println("findMains: Running", cmd, "in", cmd.Dir)
 	out, err := cmd.Output()
 	if err != nil {
 		log.Println("WARNING: In findMains:", fmt.Errorf("%q: %w", cmd.Args, err))
+		// See https://bugs.debian.org/992610
 		log.Printf("Retrying without appending \"/...\" to repo")
-		cmd = exec.Command("go", "list", "-f", "{{.ImportPath}} {{.Name}}", repo)
+		cmd = exec.Command("go", "list", "-e", "-f", "{{.ImportPath}} {{.Name}}", repo)
+		cmd.Dir = filepath.Join(gopath, "src", repo)
+		cmd.Env = passthroughEnv()
 		cmd.Stderr = os.Stderr
-		cmd.Env = append([]string{
-			"GO111MODULE=off",
-			"GOPATH=" + gopath,
-		}, passthroughEnv()...)
+		log.Println("findMains: Running", cmd, "in", cmd.Dir)
 		out, err = cmd.Output()
 		if err != nil {
-			return fmt.Errorf("%q: %w", cmd.Args, err)
+			log.Println("WARNING: In findMains:", fmt.Errorf("%q: %w", cmd.Args, err))
 		}
 	}
 	for _, line := range strings.Split(strings.TrimSpace(string(out)), "\n") {
@@ -254,26 +269,23 @@ func (u *upstream) findMains(gopath, repo string) error {
 func (u *upstream) findDependencies(gopath, repo string) error {
 	log.Printf("Determining dependencies\n")
 
-	cmd := exec.Command("go", "list", "-f", "{{join .Imports \"\\n\"}}\n{{join .TestImports \"\\n\"}}\n{{join .XTestImports \"\\n\"}}", repo+"/...")
+	cmd := exec.Command("go", "list", "-e", "-f", "{{join .Imports \"\\n\"}}\n{{join .TestImports \"\\n\"}}\n{{join .XTestImports \"\\n\"}}", repo+"/...")
+	cmd.Dir = filepath.Join(gopath, "src", repo)
+	cmd.Env = passthroughEnv()
 	cmd.Stderr = os.Stderr
-	cmd.Env = append([]string{
-		"GO111MODULE=off",
-		"GOPATH=" + gopath,
-	}, passthroughEnv()...)
 
 	out, err := cmd.Output()
 	if err != nil {
 		log.Println("WARNING: In findDependencies:", fmt.Errorf("%q: %w", cmd.Args, err))
+		// See https://bugs.debian.org/992610
 		log.Printf("Retrying without appending \"/...\" to repo")
-		cmd = exec.Command("go", "list", "-f", "{{join .Imports \"\\n\"}}\n{{join .TestImports \"\\n\"}}\n{{join .XTestImports \"\\n\"}}", repo)
+		cmd = exec.Command("go", "list", "-e", "-f", "{{join .Imports \"\\n\"}}\n{{join .TestImports \"\\n\"}}\n{{join .XTestImports \"\\n\"}}", repo)
+		cmd.Dir = filepath.Join(gopath, "src", repo)
+		cmd.Env = passthroughEnv()
 		cmd.Stderr = os.Stderr
-		cmd.Env = append([]string{
-			"GO111MODULE=off",
-			"GOPATH=" + gopath,
-		}, passthroughEnv()...)
 		out, err = cmd.Output()
 		if err != nil {
-			return fmt.Errorf("%q: %w", cmd.Args, err)
+			log.Println("WARNING: In findDependencies:", fmt.Errorf("%q: %w", cmd.Args, err))
 		}
 	}
 
@@ -299,12 +311,8 @@ func (u *upstream) findDependencies(gopath, repo string) error {
 
 	// Remove all packages which are in the standard lib.
 	cmd = exec.Command("go", "list", "std")
-	cmd.Dir = filepath.Join(gopath, "src", repo)
 	cmd.Stderr = os.Stderr
-	cmd.Env = append([]string{
-		// Not affected by GO111MODULE
-		"GOPATH=" + gopath,
-	}, passthroughEnv()...)
+	cmd.Env = passthroughEnv()
 
 	out, err = cmd.Output()
 	if err != nil {
@@ -336,7 +344,7 @@ func (u *upstream) findDependencies(gopath, repo string) error {
 }
 
 func makeUpstreamSourceTarball(repo, revision string, forcePrerelease bool) (*upstream, error) {
-	gopath, err := ioutil.TempDir("", "dh-make-golang")
+	gopath, err := os.MkdirTemp("", "dh-make-golang")
 	if err != nil {
 		return nil, fmt.Errorf("create tmp dir: %w", err)
 	}
@@ -379,7 +387,7 @@ func makeUpstreamSourceTarball(repo, revision string, forcePrerelease bool) (*up
 
 	log.Printf("Determining upstream version number\n")
 
-	u.version, err = pkgVersionFromGit(repoDir, &u, forcePrerelease)
+	u.version, err = pkgVersionFromGit(repoDir, &u, revision, forcePrerelease)
 	if err != nil {
 		return nil, fmt.Errorf("get package version from Git: %w", err)
 	}
@@ -419,18 +427,9 @@ func createGitRepository(debsrc, gopkg, orig string, u *upstream,
 		return "", fmt.Errorf("mkdir: %w", err)
 	}
 
-	// "git init -b" is the one-liner we need here, however it was added in Git 2.28.
-	// For now we prefer to keep compatibility with older Git, so we do it in two
-	// rounds, "git init" then "git checkout".
-	//if err := runGitCommandIn(dir, "init", "-b", debianBranch); err != nil {
-	//	return dir, err
-	//}
-	if err := runGitCommandIn(dir, "init"); err != nil {
+	if err := runGitCommandIn(dir, "init", "-b", debianBranch); err != nil {
 		return dir, fmt.Errorf("git init: %w", err)
 	}
-	if err := runGitCommandIn(dir, "checkout", "-q", "-b", debianBranch); err != nil {
-		return dir, fmt.Errorf("git checkout: %w", err)
-	}
 
 	// Set repository options
 
@@ -482,12 +481,15 @@ func createGitRepository(debsrc, gopkg, orig string, u *upstream,
 		if err != nil {
 			return dir, fmt.Errorf("unable to fetch upstream history: %q", err)
 		}
+		if u.remote == "debian" {
+			u.remote = "salsa"
+		}
 		log.Printf("Adding remote %q with URL %q\n", u.remote, u.rr.Repo)
 		if err := runGitCommandIn(dir, "remote", "add", u.remote, u.rr.Repo); err != nil {
 			return dir, fmt.Errorf("git remote add %s %s: %w", u.remote, u.rr.Repo, err)
 		}
-		log.Printf("Running \"git fetch %s\"\n", u.remote)
-		if err := runGitCommandIn(dir, "fetch", u.remote); err != nil {
+		log.Printf("Running \"git fetch --tags %s\"\n", u.remote)
+		if err := runGitCommandIn(dir, "fetch", "--tags", u.remote); err != nil {
 			return dir, fmt.Errorf("git fetch %s: %w", u.remote, err)
 		}
 	}
@@ -572,12 +574,16 @@ func shortHostName(gopkg string, allowUnknownHoster bool) (host string, err erro
 		"blitiri.com.ar":       "blitiri",
 		"cloud.google.com":     "googlecloud",
 		"code.google.com":      "googlecode",
+		"codeberg.org":         "codeberg",
 		"filippo.io":           "filippo",
+		"fortio.org":           "fortio",
 		"fyne.io":              "fyne",
 		"git.sr.ht":            "sourcehut",
 		"github.com":           "github",
 		"gitlab.com":           "gitlab",
+		"go.bug.st":            "bugst",
 		"go.cypherpunks.ru":    "cypherpunks",
+		"go.mongodb.org":       "mongodb",
 		"go.opentelemetry.io":  "opentelemetry",
 		"go.step.sm":           "step",
 		"go.uber.org":          "uber",
@@ -589,11 +595,13 @@ func shortHostName(gopkg string, allowUnknownHoster bool) (host string, err erro
 		"honnef.co":            "honnef",
 		"howett.net":           "howett",
 		"k8s.io":               "k8s",
+		"modernc.org":          "modernc",
 		"pault.ag":             "pault",
 		"rsc.io":               "rsc",
 		"salsa.debian.org":     "debian",
 		"sigs.k8s.io":          "k8s-sigs",
 		"software.sslmate.com": "sslmate",
+		"zgo.at":               "zgoat",
 	}
 	parts := strings.Split(gopkg, "/")
 	fqdn := parts[0]
@@ -648,7 +656,7 @@ func getDebianEmail() string {
 	if email := strings.TrimSpace(os.Getenv("DEBEMAIL")); email != "" {
 		return email
 	}
-	mailname, err := ioutil.ReadFile("/etc/mailname")
+	mailname, err := os.ReadFile("/etc/mailname")
 	// By default, /etc/mailname contains "debian" which is not useful; check for ".".
 	if err == nil && strings.Contains(string(mailname), ".") {
 		if u, err := user.Current(); err == nil && u.Username != "" {
@@ -685,9 +693,11 @@ func writeITP(gopkg, debsrc, debversion string) (string, error) {
 		description = "TODO"
 	}
 
-	fmt.Fprintf(f, "From: %q <%s>\n", getDebianName(), getDebianEmail())
+	subject := mime.QEncoding.Encode("utf-8", fmt.Sprintf("ITP: %s -- %s", debsrc, description))
+
+	fmt.Fprintf(f, "From: %q <%s>\n", mime.QEncoding.Encode("utf-8", getDebianName()), getDebianEmail())
 	fmt.Fprintf(f, "To: submit@bugs.debian.org\n")
-	fmt.Fprintf(f, "Subject: ITP: %s -- %s\n", debsrc, description)
+	fmt.Fprintf(f, "Subject: %s\n", subject)
 	fmt.Fprintf(f, "Content-Type: text/plain; charset=utf-8\n")
 	fmt.Fprintf(f, "Content-Transfer-Encoding: 8bit\n")
 	fmt.Fprintf(f, "X-Debbugs-CC: debian-devel@lists.debian.org, debian-go@lists.debian.org\n")
@@ -818,7 +828,7 @@ func execMake(args []string, usage func()) {
 
 	fs.StringVar(&wrapAndSort,
 		"wrap-and-sort",
-		"a",
+		"at",
 		"Set how the various multi-line fields in debian/control are formatted.\n"+
 			"Valid values are \"a\", \"at\" and \"ast\", see wrap-and-sort(1) man page\n"+
 			"for more information.")
@@ -923,7 +933,7 @@ func execMake(args []string, usage func()) {
 
 	var (
 		eg             errgroup.Group
-		golangBinaries map[string]string // map[goImportPath]debianBinaryPackage
+		golangBinaries map[string]debianPackage // map[goImportPath]debianPackage
 	)
 
 	// TODO: also check whether there already is a git repository on salsa.
@@ -956,9 +966,9 @@ func execMake(args []string, usage func()) {
 		log.Printf("Could not check for existing Go packages in Debian: %v", err)
 	}
 
-	if debbin, ok := golangBinaries[gopkg]; ok {
+	if debpkg, ok := golangBinaries[gopkg]; ok {
 		log.Printf("WARNING: A package called %q is already in Debian! See https://tracker.debian.org/pkg/%s\n",
-			debbin, debbin)
+			debpkg.binary, debpkg.source)
 	}
 
 	orig := fmt.Sprintf("%s_%s.orig.tar.%s", debsrc, u.version, u.compression)
@@ -986,12 +996,12 @@ func execMake(args []string, usage func()) {
 			debdependencies = append(debdependencies, debianNameFromGopkg(dep, typeLibrary, "", allowUnknownHoster)+"-dev")
 			continue
 		}
-		bin, ok := golangBinaries[dep]
+		pkg, ok := golangBinaries[dep]
 		if !ok {
 			log.Printf("Build-Dependency %q is not yet available in Debian, or has not yet been converted to use XS-Go-Import-Path in debian/control", dep)
 			continue
 		}
-		debdependencies = append(debdependencies, bin)
+		debdependencies = append(debdependencies, pkg.binary)
 	}
 
 	if err := writeTemplates(dir, gopkg, debsrc, debLib, debProg, debversion,
@@ -1041,11 +1051,6 @@ func execMake(args []string, usage func()) {
 	fmt.Printf("\n")
 
 	if includeUpstreamHistory {
-		fmt.Printf("NOTE: Full upstream git history has been included as per pkg-go team's\n")
-		fmt.Printf("      new workflow.  This feature is new and somewhat experimental,\n")
-		fmt.Printf("      and all feedback are welcome!\n")
-		fmt.Printf("      (For old behavior, use --upstream_git_history=false)\n")
-		fmt.Printf("\n")
 		fmt.Printf("The upstream git history is being tracked with the remote named %q.\n", u.remote)
 		fmt.Printf("To upgrade to the latest upstream version, you may use something like:\n")
 		fmt.Printf("    git fetch %-15v # note the latest tag or commit-ish\n", u.remote)
